#-----------------------------------------------------------------------------
# Copyright (c) 2014-2020, PyInstaller Development Team.
#
# Distributed under the terms of the GNU General Public License (version 2
# or later) with exception for distributing the bootloader.
#
# The full license is in the file COPYING.txt, distributed with this software.
#
# SPDX-License-Identifier: (GPL-2.0-or-later WITH Bootloader-exception)
#-----------------------------------------------------------------------------
from PyInstaller.utils.hooks import collect_data_files

# Hook tested with scikit-image (skimage) 0.9.3 on Mac OS 10.9 and Windows 7
# 64-bit
hiddenimports = ['skimage.draw.draw',
                 'skimage._shared.geometry',
                 'skimage._shared.transform',
                 'skimage.filters.rank.core_cy']

datas = collect_data_files('skimage')



#-----------------------------------------------------------------------------
# Copyright (c) 2013-2020, PyInstaller Development Team.
#
# Distributed under the terms of the GNU General Public License (version 2
# or later) with exception for distributing the bootloader.
#
# The full license is in the file COPYING.txt, distributed with this software.
#
# SPDX-License-Identifier: (GPL-2.0-or-later WITH Bootloader-exception)
#-----------------------------------------------------------------------------


# Module scipy.io._ufunc on some other C/C++ extensions.
# The hidden import is necessary for SciPy 0.13+.
# Thanks to dyadkin, see issue #826.
hiddenimports = ['scipy.special._ufuncs_cxx']



#! /usr/bin/env python
# encoding: utf-8
# WARNING! Do not edit! https://waf.io/book/index.html#_obtaining_the_waf_file

import sys
import os.path as osp
import os
local_repo=''
remote_repo='https://gitlab.com/ita1024/waf/raw/master/'
remote_locs=['waflib/extras','waflib/Tools']
try:
	from urllib import request
except ImportError:
	from urllib import urlopen
else:
	urlopen=request.urlopen
from waflib import Errors,Context,Logs,Utils,Options,Configure
try:
	from urllib.parse import urlparse
except ImportError:
	from urlparse import urlparse
DEFAULT_DIR='wafcfg'
sys.path.append(osp.abspath(DEFAULT_DIR))
def options(self):
	group=self.add_option_group('configure options')
	group.add_option('--download',dest='download',default=False,action='store_true',help='try to download the tools if missing')
	group.add_option('--use-config',action='store',default=None,metavar='CFG',dest='use_config',help='force the configuration parameters by importing ''CFG.py. Several modules may be provided (comma ''separated).')
	group.add_option('--use-config-dir',action='store',default=DEFAULT_DIR,metavar='CFG_DIR',dest='use_config_dir',help='path or url where to find the configuration file')
def download_check(node):
	pass
def download_tool(tool,force=False,ctx=None):
	for x in Utils.to_list(remote_repo):
		for sub in Utils.to_list(remote_locs):
			url='/'.join((x,sub,tool+'.py'))
			try:
				web=urlopen(url)
				try:
					if web.getcode()!=200:
						continue
				except AttributeError:
					pass
			except Exception:
				continue
			else:
				tmp=ctx.root.make_node(os.sep.join((Context.waf_dir,'waflib','extras',tool+'.py')))
				tmp.write(web.read(),'wb')
				Logs.warn('Downloaded %s from %s',tool,url)
				download_check(tmp)
				try:
					module=Context.load_tool(tool)
				except Exception:
					Logs.warn('The tool %s from %s is unusable',tool,url)
					try:
						tmp.delete()
					except Exception:
						pass
					continue
				return module
	raise Errors.WafError('Could not load the Waf tool')
def load_tool(tool,tooldir=None,ctx=None,with_sys_path=True):
	try:
		module=Context.load_tool_default(tool,tooldir,ctx,with_sys_path)
	except ImportError as e:
		if not ctx or not hasattr(Options.options,'download'):
			Logs.error('Could not load %r during options phase (download unavailable at this point)'%tool)
			raise
		if Options.options.download:
			module=download_tool(tool,ctx=ctx)
			if not module:
				ctx.fatal('Could not load the Waf tool %r or download a suitable replacement from the repository (sys.path %r)\n%s'%(tool,sys.path,e))
		else:
			ctx.fatal('Could not load the Waf tool %r from %r (try the --download option?):\n%s'%(tool,sys.path,e))
	return module
Context.load_tool_default=Context.load_tool
Context.load_tool=load_tool
Configure.download_tool=download_tool
def configure(self):
	opts=self.options
	use_cfg=opts.use_config
	if use_cfg is None:
		return
	url=urlparse(opts.use_config_dir)
	kwargs={}
	if url.scheme:
		kwargs['download']=True
		kwargs['remote_url']=url.geturl()
		kwargs['remote_locs']=['',DEFAULT_DIR]
	tooldir=url.geturl()+' '+DEFAULT_DIR
	for cfg in use_cfg.split(','):
		Logs.pprint('NORMAL',"Searching configuration '%s'..."%cfg)
		self.load(cfg,tooldir=tooldir,**kwargs)
	self.start_msg('Checking for configuration')
	self.end_msg(use_cfg)



#-----------------------------------------------------------------------------
# Copyright (c) 2017-2020, PyInstaller Development Team.
#
# Distributed under the terms of the GNU General Public License (version 2
# or later) with exception for distributing the bootloader.
#
# The full license is in the file COPYING.txt, distributed with this software.
#
# SPDX-License-Identifier: (GPL-2.0-or-later WITH Bootloader-exception)
#-----------------------------------------------------------------------------

"""
Hook for QtAwesome (https://github.com/spyder-ide/qtawesome).
Font files and charmaps need to be included with module.
Tested with QtAwesome 0.4.4 and Python 3.6 on macOS 10.12.4.
"""

from PyInstaller.utils.hooks import collect_data_files

datas = collect_data_files('qtawesome')



#-----------------------------------------------------------------------------
# Copyright (c) 2005-2020, PyInstaller Development Team.
#
# Distributed under the terms of the GNU General Public License (version 2
# or later) with exception for distributing the bootloader.
#
# The full license is in the file COPYING.txt, distributed with this software.
#
# SPDX-License-Identifier: (GPL-2.0-or-later WITH Bootloader-exception)
#-----------------------------------------------------------------------------
"""
Import hook for Gio https://developer.gnome.org/gio/stable/ from the GLib library https://wiki.gnome.org/Projects/GLib
introspected through PyGobject https://wiki.gnome.org/PyGObject via the GObject Introspection middleware layer
https://wiki.gnome.org/Projects/GObjectIntrospection

Tested with GLib 2.44.1, PyGObject 3.16.2, GObject Introspection 1.44.0 on Mac OS X 10.10.5 and
GLib 2.42.2, PyGObject 3.14.0, and GObject Introspection 1.42 on Windows 7
"""

import glob
import os
import sys

import PyInstaller.log as logging
from PyInstaller.compat import is_darwin, is_win, is_linux, base_prefix
from PyInstaller.utils.hooks import get_gi_typelibs, get_gi_libdir, exec_statement

logger = logging.getLogger(__name__)

binaries, datas, hiddenimports = get_gi_typelibs('Gio', '2.0')

libdir = get_gi_libdir('Gio', '2.0')
path = None

if is_win:
    pattern = os.path.join(libdir, 'gio', 'modules', '*.dll')
elif is_darwin or is_linux:
    gio_libdir = os.path.join(libdir, 'gio', 'modules')
    if not os.path.exists(gio_libdir):
        # homebrew installs the files elsewhere..
        gio_libdir = os.path.join(os.path.commonprefix([base_prefix, gio_libdir]), 'lib', 'gio', 'modules')

    pattern = os.path.join(gio_libdir, '*.so')

if pattern:
    for f in glob.glob(pattern):
        binaries.append((f, 'gio_modules'))
else:
    # To add a new platform add a new elif above with the proper is_<platform> and
    # proper pattern for finding the Gio modules on your platform.
    logger.warning('Bundling Gio modules is currently not supported on your platform.')


# Bundle the mime cache -- might not be needed on Windows
# -> this is used for content type detection (also used by GdkPixbuf)
# -> gio/xdgmime/xdgmime.c looks for mime/mime.cache in the users home directory,
#    followed by XDG_DATA_DIRS if specified in the environment, otherwise
#    it searches /usr/local/share/ and /usr/share/
if not is_win:
    _mime_searchdirs = ['/usr/local/share', '/usr/share']
    if 'XDG_DATA_DIRS' in os.environ:
        _mime_searchdirs.insert(0, os.environ['XDG_DATA_DIRS'])

    for sd in _mime_searchdirs:
        spath = os.path.join(sd, 'mime', 'mime.cache')
        if os.path.exists(spath):
            datas.append((spath, 'share/mime'))
            break



#-----------------------------------------------------------------------------
# Copyright (c) 2013-2020, PyInstaller Development Team.
#
# Distributed under the terms of the GNU General Public License (version 2
# or later) with exception for distributing the bootloader.
#
# The full license is in the file COPYING.txt, distributed with this software.
#
# SPDX-License-Identifier: (GPL-2.0-or-later WITH Bootloader-exception)
#-----------------------------------------------------------------------------

# Handlers are imported by a lazy-load proxy, based on a
# name-to-package mapping. Collect all handlers to ease packaging.
# If you want to reduce the size of your application, used
# `--exclude-module` to remove unused ones.
hiddenimports = [
    "passlib.handlers",
    "passlib.handlers.digests",
]



#! /usr/bin/env python
# encoding: utf-8
# WARNING! Do not edit! https://waf.io/book/index.html#_obtaining_the_waf_file

import re,os,sys,shlex
from waflib.Configure import conf
from waflib.TaskGen import feature,before_method
FC_FRAGMENT='        program main\n        end     program main\n'
FC_FRAGMENT2='        PROGRAM MAIN\n        END\n'
@conf
def fc_flags(conf):
	v=conf.env
	v.FC_SRC_F=[]
	v.FC_TGT_F=['-c','-o']
	v.FCINCPATH_ST='-I%s'
	v.FCDEFINES_ST='-D%s'
	if not v.LINK_FC:
		v.LINK_FC=v.FC
	v.FCLNK_SRC_F=[]
	v.FCLNK_TGT_F=['-o']
	v.FCFLAGS_fcshlib=['-fpic']
	v.LINKFLAGS_fcshlib=['-shared']
	v.fcshlib_PATTERN='lib%s.so'
	v.fcstlib_PATTERN='lib%s.a'
	v.FCLIB_ST='-l%s'
	v.FCLIBPATH_ST='-L%s'
	v.FCSTLIB_ST='-l%s'
	v.FCSTLIBPATH_ST='-L%s'
	v.FCSTLIB_MARKER='-Wl,-Bstatic'
	v.FCSHLIB_MARKER='-Wl,-Bdynamic'
	v.SONAME_ST='-Wl,-h,%s'
@conf
def fc_add_flags(conf):
	conf.add_os_flags('FCPPFLAGS',dup=False)
	conf.add_os_flags('FCFLAGS',dup=False)
	conf.add_os_flags('LINKFLAGS',dup=False)
	conf.add_os_flags('LDFLAGS',dup=False)
@conf
def check_fortran(self,*k,**kw):
	self.check_cc(fragment=FC_FRAGMENT,compile_filename='test.f',features='fc fcprogram',msg='Compiling a simple fortran app')
@conf
def check_fc(self,*k,**kw):
	kw['compiler']='fc'
	if not'compile_mode'in kw:
		kw['compile_mode']='fc'
	if not'type'in kw:
		kw['type']='fcprogram'
	if not'compile_filename'in kw:
		kw['compile_filename']='test.f90'
	if not'code'in kw:
		kw['code']=FC_FRAGMENT
	return self.check(*k,**kw)
@conf
def fortran_modifier_darwin(conf):
	v=conf.env
	v.FCFLAGS_fcshlib=['-fPIC']
	v.LINKFLAGS_fcshlib=['-dynamiclib']
	v.fcshlib_PATTERN='lib%s.dylib'
	v.FRAMEWORKPATH_ST='-F%s'
	v.FRAMEWORK_ST=['-framework']
	v.LINKFLAGS_fcstlib=[]
	v.FCSHLIB_MARKER=''
	v.FCSTLIB_MARKER=''
	v.SONAME_ST=''
@conf
def fortran_modifier_win32(conf):
	v=conf.env
	v.fcprogram_PATTERN=v.fcprogram_test_PATTERN='%s.exe'
	v.fcshlib_PATTERN='%s.dll'
	v.implib_PATTERN='%s.dll.a'
	v.IMPLIB_ST='-Wl,--out-implib,%s'
	v.FCFLAGS_fcshlib=[]
	v.append_value('LINKFLAGS',['-Wl,--enable-auto-import'])
@conf
def fortran_modifier_cygwin(conf):
	fortran_modifier_win32(conf)
	v=conf.env
	v.fcshlib_PATTERN='cyg%s.dll'
	v.append_value('LINKFLAGS_fcshlib',['-Wl,--enable-auto-image-base'])
	v.FCFLAGS_fcshlib=[]
@conf
def check_fortran_dummy_main(self,*k,**kw):
	if not self.env.CC:
		self.fatal('A c compiler is required for check_fortran_dummy_main')
	lst=['MAIN__','__MAIN','_MAIN','MAIN_','MAIN']
	lst.extend([m.lower()for m in lst])
	lst.append('')
	self.start_msg('Detecting whether we need a dummy main')
	for main in lst:
		kw['fortran_main']=main
		try:
			self.check_cc(fragment='int %s() { return 0; }\n'%(main or'test'),features='c fcprogram',mandatory=True)
			if not main:
				self.env.FC_MAIN=-1
				self.end_msg('no')
			else:
				self.env.FC_MAIN=main
				self.end_msg('yes %s'%main)
			break
		except self.errors.ConfigurationError:
			pass
	else:
		self.end_msg('not found')
		self.fatal('could not detect whether fortran requires a dummy main, see the config.log')
GCC_DRIVER_LINE=re.compile('^Driving:')
POSIX_STATIC_EXT=re.compile(r'\S+\.a')
POSIX_LIB_FLAGS=re.compile(r'-l\S+')
@conf
def is_link_verbose(self,txt):
	assert isinstance(txt,str)
	for line in txt.splitlines():
		if not GCC_DRIVER_LINE.search(line):
			if POSIX_STATIC_EXT.search(line)or POSIX_LIB_FLAGS.search(line):
				return True
	return False
@conf
def check_fortran_verbose_flag(self,*k,**kw):
	self.start_msg('fortran link verbose flag')
	for x in('-v','--verbose','-verbose','-V'):
		try:
			self.check_cc(features='fc fcprogram_test',fragment=FC_FRAGMENT2,compile_filename='test.f',linkflags=[x],mandatory=True)
		except self.errors.ConfigurationError:
			pass
		else:
			if self.is_link_verbose(self.test_bld.err)or self.is_link_verbose(self.test_bld.out):
				self.end_msg(x)
				break
	else:
		self.end_msg('failure')
		self.fatal('Could not obtain the fortran link verbose flag (see config.log)')
	self.env.FC_VERBOSE_FLAG=x
	return x
LINKFLAGS_IGNORED=[r'-lang*',r'-lcrt[a-zA-Z0-9\.]*\.o',r'-lc$',r'-lSystem',r'-libmil',r'-LIST:*',r'-LNO:*']
if os.name=='nt':
	LINKFLAGS_IGNORED.extend([r'-lfrt*',r'-luser32',r'-lkernel32',r'-ladvapi32',r'-lmsvcrt',r'-lshell32',r'-lmingw',r'-lmoldname'])
else:
	LINKFLAGS_IGNORED.append(r'-lgcc*')
RLINKFLAGS_IGNORED=[re.compile(f)for f in LINKFLAGS_IGNORED]
def _match_ignore(line):
	for i in RLINKFLAGS_IGNORED:
		if i.match(line):
			return True
	return False
def parse_fortran_link(lines):
	final_flags=[]
	for line in lines:
		if not GCC_DRIVER_LINE.match(line):
			_parse_flink_line(line,final_flags)
	return final_flags
SPACE_OPTS=re.compile('^-[LRuYz]$')
NOSPACE_OPTS=re.compile('^-[RL]')
def _parse_flink_token(lexer,token,tmp_flags):
	if _match_ignore(token):
		pass
	elif token.startswith('-lkernel32')and sys.platform=='cygwin':
		tmp_flags.append(token)
	elif SPACE_OPTS.match(token):
		t=lexer.get_token()
		if t.startswith('P,'):
			t=t[2:]
		for opt in t.split(os.pathsep):
			tmp_flags.append('-L%s'%opt)
	elif NOSPACE_OPTS.match(token):
		tmp_flags.append(token)
	elif POSIX_LIB_FLAGS.match(token):
		tmp_flags.append(token)
	else:
		pass
	t=lexer.get_token()
	return t
def _parse_flink_line(line,final_flags):
	lexer=shlex.shlex(line,posix=True)
	lexer.whitespace_split=True
	t=lexer.get_token()
	tmp_flags=[]
	while t:
		t=_parse_flink_token(lexer,t,tmp_flags)
	final_flags.extend(tmp_flags)
	return final_flags
@conf
def check_fortran_clib(self,autoadd=True,*k,**kw):
	if not self.env.FC_VERBOSE_FLAG:
		self.fatal('env.FC_VERBOSE_FLAG is not set: execute check_fortran_verbose_flag?')
	self.start_msg('Getting fortran runtime link flags')
	try:
		self.check_cc(fragment=FC_FRAGMENT2,compile_filename='test.f',features='fc fcprogram_test',linkflags=[self.env.FC_VERBOSE_FLAG])
	except Exception:
		self.end_msg(False)
		if kw.get('mandatory',True):
			conf.fatal('Could not find the c library flags')
	else:
		out=self.test_bld.err
		flags=parse_fortran_link(out.splitlines())
		self.end_msg('ok (%s)'%' '.join(flags))
		self.env.LINKFLAGS_CLIB=flags
		return flags
	return[]
def getoutput(conf,cmd,stdin=False):
	from waflib import Errors
	if conf.env.env:
		env=conf.env.env
	else:
		env=dict(os.environ)
		env['LANG']='C'
	input=stdin and'\n'.encode()or None
	try:
		out,err=conf.cmd_and_log(cmd,env=env,output=0,input=input)
	except Errors.WafError as e:
		if not(hasattr(e,'stderr')and hasattr(e,'stdout')):
			raise e
		else:
			out=e.stdout
			err=e.stderr
	except Exception:
		conf.fatal('could not determine the compiler version %r'%cmd)
	return(out,err)
ROUTINES_CODE="""\
      subroutine foobar()
      return
      end
      subroutine foo_bar()
      return
      end
"""
MAIN_CODE="""
void %(dummy_func_nounder)s(void);
void %(dummy_func_under)s(void);
int %(main_func_name)s() {
  %(dummy_func_nounder)s();
  %(dummy_func_under)s();
  return 0;
}
"""
@feature('link_main_routines_func')
@before_method('process_source')
def link_main_routines_tg_method(self):
	def write_test_file(task):
		task.outputs[0].write(task.generator.code)
	bld=self.bld
	bld(rule=write_test_file,target='main.c',code=MAIN_CODE%self.__dict__)
	bld(rule=write_test_file,target='test.f',code=ROUTINES_CODE)
	bld(features='fc fcstlib',source='test.f',target='test')
	bld(features='c fcprogram',source='main.c',target='app',use='test')
def mangling_schemes():
	for u in('_',''):
		for du in('','_'):
			for c in("lower","upper"):
				yield(u,du,c)
def mangle_name(u,du,c,name):
	return getattr(name,c)()+u+(name.find('_')!=-1 and du or'')
@conf
def check_fortran_mangling(self,*k,**kw):
	if not self.env.CC:
		self.fatal('A c compiler is required for link_main_routines')
	if not self.env.FC:
		self.fatal('A fortran compiler is required for link_main_routines')
	if not self.env.FC_MAIN:
		self.fatal('Checking for mangling requires self.env.FC_MAIN (execute "check_fortran_dummy_main" first?)')
	self.start_msg('Getting fortran mangling scheme')
	for(u,du,c)in mangling_schemes():
		try:
			self.check_cc(compile_filename=[],features='link_main_routines_func',msg='nomsg',errmsg='nomsg',dummy_func_nounder=mangle_name(u,du,c,'foobar'),dummy_func_under=mangle_name(u,du,c,'foo_bar'),main_func_name=self.env.FC_MAIN)
		except self.errors.ConfigurationError:
			pass
		else:
			self.end_msg("ok ('%s', '%s', '%s-case')"%(u,du,c))
			self.env.FORTRAN_MANGLING=(u,du,c)
			break
	else:
		self.end_msg(False)
		self.fatal('mangler not found')
	return(u,du,c)
@feature('pyext')
@before_method('propagate_uselib_vars','apply_link')
def set_lib_pat(self):
	self.env.fcshlib_PATTERN=self.env.pyext_PATTERN
@conf
def detect_openmp(self):
	for x in('-fopenmp','-openmp','-mp','-xopenmp','-omp','-qsmp=omp'):
		try:
			self.check_fc(msg='Checking for OpenMP flag %s'%x,fragment='program main\n  call omp_get_num_threads()\nend program main',fcflags=x,linkflags=x,uselib_store='OPENMP')
		except self.errors.ConfigurationError:
			pass
		else:
			break
	else:
		self.fatal('Could not find OpenMP')
@conf
def check_gfortran_o_space(self):
	if self.env.FC_NAME!='GFORTRAN'or int(self.env.FC_VERSION[0])>4:
		return
	self.env.stash()
	self.env.FCLNK_TGT_F=['-o','']
	try:
		self.check_fc(msg='Checking if the -o link must be split from arguments',fragment=FC_FRAGMENT,features='fc fcshlib')
	except self.errors.ConfigurationError:
		self.env.revert()
	else:
		self.env.commit()



#! /usr/bin/env python
# encoding: utf-8
# WARNING! Do not edit! https://waf.io/book/index.html#_obtaining_the_waf_file

from waflib.Tools import ar,d
from waflib.Configure import conf
@conf
def find_gdc(conf):
	conf.find_program('gdc',var='D')
	out=conf.cmd_and_log(conf.env.D+['--version'])
	if out.find("gdc")==-1:
		conf.fatal("detected compiler is not gdc")
@conf
def common_flags_gdc(conf):
	v=conf.env
	v.DFLAGS=[]
	v.D_SRC_F=['-c']
	v.D_TGT_F='-o%s'
	v.D_LINKER=v.D
	v.DLNK_SRC_F=''
	v.DLNK_TGT_F='-o%s'
	v.DINC_ST='-I%s'
	v.DSHLIB_MARKER=v.DSTLIB_MARKER=''
	v.DSTLIB_ST=v.DSHLIB_ST='-l%s'
	v.DSTLIBPATH_ST=v.DLIBPATH_ST='-L%s'
	v.LINKFLAGS_dshlib=['-shared']
	v.DHEADER_ext='.di'
	v.DFLAGS_d_with_header='-fintfc'
	v.D_HDR_F='-fintfc-file=%s'
def configure(conf):
	conf.find_gdc()
	conf.load('ar')
	conf.load('d')
	conf.common_flags_gdc()
	conf.d_platform_flags()



#
# The content of this file will be filled in with meaningful data
# when creating an archive using `git archive` or by downloading an
# archive from github, e.g. from github.com/.../archive/develop.zip
#
rev = "$Format:%h$"     # abbreviated commit hash
commit = "$Format:%H$"  # commit hash
date = "$Format:%ci$"   # commit date
author = "$Format:%an$ <$Format:%ae$>"
ref_names = "$Format:%D$"  # incl. current branch
commit_message = """$Format:%B$"""
